use crate::vm::implementation::DvmImplementation;
use crate::vm::method::DvmMethod;
use crate::vm::value::DvmValue;
use std::rc::Rc;

#[derive(Debug, PartialEq)]
pub struct DvmObject {
    data: Vec<DvmValue>,
    implementation: Rc<DvmImplementation>,
}

impl DvmObject {
    pub fn new(implementation: Rc<DvmImplementation>) -> Self {
        DvmObject {
            data: vec![DvmValue::Empty; implementation.field_count()],
            implementation,
        }
    }

    fn get_field_index(&self, field_name: &str) -> usize {
        self.implementation
            .find_field(field_name, self)
            .unwrap()
            .index()
    }

    pub fn data(&self) -> &[DvmValue] {
        &self.data
    }

    pub fn implementation(&self) -> &DvmImplementation {
        &self.implementation
    }

    pub fn read_field(&self, field_name: &str) -> &DvmValue {
        &self.data[self.get_field_index(field_name)]
    }

    pub fn take_field(&mut self, field_name: &str) -> DvmValue {
        let field_index = self.get_field_index(field_name);
        std::mem::take(&mut self.data[field_index])
    }

    pub fn write_field(&mut self, field_name: &str, value: DvmValue) {
        let field_index = self.get_field_index(field_name);
        self.data[field_index] = value;
    }

    pub fn get_method(&self, method_name: &str) -> Option<Rc<DvmMethod>> {
        self.implementation.find_method(method_name, self)
    }
}
